#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HSPyLib
   @package: hspylib.main.hspylib.modules.cli.icons.font_awesome
      @file: widget_icons.py
   @created: Thu, 20 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright 2021, HSPyLib team
"""

from hspylib.modules.cli.icons.font_awesome.awesome import Awesome


class WidgetIcons(Awesome):
    """
        Dashboard UI icons.
        Codes can be found here:
        - https://fontawesome.com/cheatsheet?from=io
    """

    # @formatter:off
    WIDGET = u'\uFC65'    # ﱥ
    DATABASE = u'\uFB19'  # ﬙
    CLOCK = u'\uF651'     # 
    NETWORK = u'\uF819'   # 
    CHART_1 = u'\uFC67'   # ﱨ
    CHART_2 = u'\uFC68'   # ﱩ
    MUSIC = u'\uFC6E'     # ﱯ
    SIGN = u'\uFC7E'      # ﱿ
    SWORDS = u'\uFC84'    # ﲅ
    # @formatter:on
