# _*_ coding: utf-8 _*_
#
# HSPyLib v0.11.1
#
# Package: main.hspylib.modules.cli.tui.extra.mdashboard
"""Package initialization."""

__all__ = [
  'dashboard_builder', 
  'dashboard_item', 
  'mdashboard'
]
