# _*_ coding: utf-8 _*_
#
# HSPyLib v0.11.1
#
# Package: main.hspylib.core.crud
"""Package initialization."""

__all__ = [
  'crud_entity', 
  'crud_repository', 
  'crud_service', 
  'db', 
  'file'
]
